part of '../bolter_provider.dart';

abstract class Presenter<P extends Presenter<P>> {
  late Bolter _bolter;
  late BuildContext context;
  var _disposed = false;

  bool get disposed => _disposed;

  late final itSelf = this as P;

  @protected
  ValueStream<V> stream<V>(V Function() getter, {bool distinct = true}) =>
      _bolter.stream(getter, distinct: distinct);

  @protected
  FutureOr<void> onLayout() {}

  @protected
  void initState() {}

  @protected
  Future<void> runAndUpdate<T>({
    void Function()? beforeAction,
    FutureOr<T> Function()? action,
    void Function()? afterAction,
    void Function(Object e)? onError,
  }) async {
    if (beforeAction != null) {
      beforeAction();
      _bolter.shake();
    }
    try {
      await action?.call();
      _bolter.shake();
    } catch (e) {
      if (onError != null) {
        onError(e);
        _bolter.shake();
      }
      rethrow;
    }
    if (afterAction != null) {
      afterAction();
      _bolter.shake();
    }
  }

  @protected
  @mustCallSuper
  FutureOr<void> dispose() {
    _disposed = true;
  }
}
