part of '../bolter_provider.dart';

class PresenterProvider<P extends Presenter<P>> extends StatefulWidget {
  final P presenter;
  final Widget child;

  const PresenterProvider({
    Key? key,
    required this.child,
    required this.presenter,
  }) : super(key: key);

  static P of<P extends Presenter<P>>(BuildContext context) {
    return context.findAncestorStateOfType<PresenterProviderState<P>>()!.presenter;
  }

  @override
  PresenterProviderState<P> createState() => PresenterProviderState<P>();
}

class PresenterProviderState<P extends Presenter<P>> extends State<PresenterProvider<P>> {
  late final P presenter = widget.presenter;

  @override
  void initState() {
    super.initState();
    presenter.context = context;
    WidgetsBinding.instance.endOfFrame.then(
      (_) {
        if (mounted) {
          presenter.onLayout();
        }
      },
    );
    presenter._bolter = BolterProviderOld._bolter;
    presenter.initState();
  }

  @override
  Widget build(BuildContext context) {
    return widget.child;
  }

  @override
  void dispose() {
    presenter.dispose();
    super.dispose();
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter<P>>() => PresenterProvider.of<P>(this);
}
