part of 'bolter_provider.dart';

class SyncBuilderController<T> {
  final BolterInterface bolter;
  late T _value;
  final _notifications = <BolterNotification>[];

  SyncBuilderController(this.bolter, this._value);

  T get value => _value;

  void update(T Function(T value) action) {
    final now = DateTime.now().millisecondsSinceEpoch;
    for (final notification in _notifications) {
      bolter.runAndUpdate(
        action: () {
          print(_value);
          _value = action(this._value);
          print(_value);
        },
        exactNotification: notification,
      );
    }
    if (kProfileBolterPerformanceLogging) {
      print('Exact notification took ${DateTime.now().millisecondsSinceEpoch - now} '
          'milliseconds');
    }
  }

  void dispose() {
    _notifications.clear();
  }
}

class SyncBuilder<T> extends StatefulWidget {
  final T Function()? getter;
  final Widget Function(BuildContext context, T value) builder;
  final SyncBuilderController? controller;

  const SyncBuilder.getter({
    Key? key,
    required this.getter,
    required this.builder,
  })  : controller = null,
        super(key: key);

  const SyncBuilder.controller({
    Key? key,
    required this.controller,
    required this.builder,
  })  : getter = null,
        super(key: key);

  @override
  _SyncBuilderState<T> createState() => _SyncBuilderState<T>();
}

class _SyncBuilderState<T> extends State<SyncBuilder<T>> {
  late final bolter =
      context.findAncestorWidgetOfExactType<BolterProvider>()?.bolter ?? defaultBolter;

  void _notification() {
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    widget.controller?._notifications.add(_notification);
    bolter.listen(widget.getter ?? () => widget.controller!._value, _notification);
  }

  @override
  void didUpdateWidget(SyncBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.getter != widget.getter || oldWidget.controller != widget.controller) {
      bolter.stopListen(_notification);
      bolter.listen(widget.getter ?? () => widget.controller!._value, _notification);
    }
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(
        context, widget.getter != null ? widget.getter!() : widget.controller!._value);
  }

  @override
  void dispose() {
    widget.controller?._notifications.remove(_notification);
    bolter.stopListen(_notification);
    super.dispose();
  }
}
