import 'dart:async';
import 'package:bolter/bolter.dart';
import 'package:flutter/material.dart';

class ValueStreamBuilder<T> extends StatefulWidget {
  final ValueStream<T> stream;
  final Widget Function(BuildContext context, T value) builder;
  final Widget Function(BuildContext context, Object error)? errorBuilder;

  const ValueStreamBuilder({
    Key? key,
    required this.stream,
    required this.builder,
    this.errorBuilder,
  }) : super(key: key);

  @override
  _ValueStreamBuilderState<T> createState() => _ValueStreamBuilderState<T>();
}

class _ValueStreamBuilderState<T> extends State<ValueStreamBuilder<T>> {
  late StreamSubscription<T> _subscription;
  late T _summary;
  Object? _error;

  @override
  void initState() {
    super.initState();
    _summary = widget.stream.value;
    _subscribe();
  }

  @override
  void didUpdateWidget(ValueStreamBuilder<T> oldWidget) {
    _summary = widget.stream.value;
    super.didUpdateWidget(oldWidget);
    if (oldWidget.stream.stream != widget.stream.stream) {
      _unsubscribe();
      _subscribe();
    }
  }

  @override
  Widget build(BuildContext context) {
    final error = _error;
    if (error != null) {
      final errorWidget = widget.errorBuilder?.call(context, error) ??
          const Text("Your error could be here");
      return errorWidget;
    }
    return widget.builder(context, _summary);
  }

  @override
  void dispose() {
    _unsubscribe();
    super.dispose();
  }

  void _subscribe() {
    _subscription = widget.stream.listen(
      (T data) {
        setState(() {
          _error = null;
          _summary = data;
        });
      },
      onError: (Object error) {
        setState(() {
          _error = error;
        });
      },
    );
  }

  void _unsubscribe() {
    _subscription.cancel();
  }
}
